local AddonName, GGL = ...

local AceGUI = LibStub("AceGUI-3.0")
local Templates = GGL.Templates
local L = GGL.Locale

local GoldGoblinNet = {}
GGL.GUI.GoldGoblinNet = GoldGoblinNet

-- lua

-- wow

-- GGL
local BNET_STREAMER_LIST = GGL.BNET_STREAMER_LIST

-- ##################
-- QuickSettings
-- ##################
local QUICK_SETTINGS_FRAME_DEFAULTS = {
    MIN_WIDTH = 250,
    MAX_WIDTH = 250,
    MIN_HEIGHT = 150,
    MAX_HEIGHT = 150,
}

local QUICK_SETTINGS_FRAME_FUNCTIONS = {

}

local function QS_Name_OnEnterPressed(self, event, text)
    text = GGL.FormatDisplayName(text)
    GGL.db.Settings.Main.displayName = text
    GGL.Templates.BNWoWPlayer:GetSelfPlayer():SetDisplayName(text)
    self:SetText(text)
    self:ClearFocus()
end

local function QS_OkButton_OnClick(self, event, mb)
    self.obj:Hide()
end

local function RewardCheckButton_OnValueChanged(self, event, state)
    local rewardID = self:GetUserData("id")
    if GGL.db.Settings.Main.rewardID == rewardID then return end
    GGL.db.Settings.Main.rewardID = rewardID
    GGL.Templates.BNWoWPlayer:GetSelfPlayer():SetGGRewardID(rewardID)
    -- update
    GoldGoblinNet.quickSettingsFrame.petReward:SetValue(rewardID == 1)
    GoldGoblinNet.quickSettingsFrame.pointsReward:SetValue(rewardID == 2)
end

local function Create_QuickSettings_Frame(self)
    if not self.quickSettingsFrame then
        local window = AceGUI:Create("Window")
        window:SetTitle("GoldGoblinLoot - "..L["Quick Settings"])

        local frame = window.frame
        local content = window.content

        -- some frame settings
        frame:SetMinResize(QUICK_SETTINGS_FRAME_DEFAULTS.MIN_WIDTH, QUICK_SETTINGS_FRAME_DEFAULTS.MIN_HEIGHT)
        frame:SetMaxResize(QUICK_SETTINGS_FRAME_DEFAULTS.MAX_WIDTH, QUICK_SETTINGS_FRAME_DEFAULTS.MAX_HEIGHT)
        frame:SetClampedToScreen(true)
        frame:SetToplevel(false)
        frame:SetFrameStrata("MEDIUM")
        --window settings
        window:SetLayout("Flow")
        window:SetWidth(QUICK_SETTINGS_FRAME_DEFAULTS.MIN_WIDTH)
        window:SetHeight(QUICK_SETTINGS_FRAME_DEFAULTS.MIN_HEIGHT)
        -- remove resizer lines
        window.line1:Hide()
        window.line2:Hide()

        local name = AceGUI:Create("EditBox")
        name:SetFullWidth(true)
        name:SetLabel(L["Twitch Name"])
        name:SetText(GGL.Templates.BNWoWPlayer:GetSelfPlayer().displayName)
        name:SetMaxLetters(GGL.MAX_NAME_LENGTH)
        name:SetCallback("OnEnterPressed", QS_Name_OnEnterPressed)
        window:AddChild(name)

        local petReward = AceGUI:Create("CheckBox")
        petReward:SetRelativeWidth(0.5)
        petReward:SetLabel(L["Pet"])
        petReward:SetUserData("id", 1)
        petReward:SetValue(GGL.db.Settings.Main.rewardID == 1)
        petReward:SetCallback("OnValueChanged", RewardCheckButton_OnValueChanged)
        window:AddChild(petReward)
        window.petReward = petReward

        local pointsReward = AceGUI:Create("CheckBox")
        pointsReward:SetRelativeWidth(0.5)
        pointsReward:SetLabel(L["Points"])
        pointsReward:SetUserData("id", 2)
        pointsReward:SetValue(GGL.db.Settings.Main.rewardID == 2)
        pointsReward:SetCallback("OnValueChanged", RewardCheckButton_OnValueChanged)
        window:AddChild(pointsReward)
        window.pointsReward = pointsReward


        local okButton = AceGUI:Create("Button")
        okButton:SetText(L["Ok"])
        okButton:SetFullWidth(true)
        okButton:SetCallback("OnClick", QS_OkButton_OnClick)
        window:AddChild(okButton)
        okButton.obj = window
        window.okButton = okButton


        -- var
        window.itemList = {}
        window.contentList = {}

        -- register functions
        for k,v in pairs(QUICK_SETTINGS_FRAME_FUNCTIONS) do
            window[k] = v
        end
        QUICK_SETTINGS_FRAME_FUNCTIONS = nil

        self.quickSettingsFrame = window
    end

    if not GGL.db.Settings.Main.rewardID or GGL.db.Settings.Main.rewardID == 0 then
        GGL.db.Settings.Main.rewardID = 2
        GGL.Templates.BNWoWPlayer:GetSelfPlayer():SetGGRewardID(2)
    end

    local frame = GoldGoblinNet.quickSettingsFrame
    frame.petReward:SetValue(GGL.db.Settings.Main.rewardID == 1)
    frame.pointsReward:SetValue(GGL.db.Settings.Main.rewardID == 2)

    GGL.Templates.BNWoWPlayer:GetSelfPlayer():SetDisplayName(GGL.db.Settings.Main.displayName)

    return self.quickSettingsFrame
end

function GoldGoblinNet:ShowQuickSettings()
    Create_QuickSettings_Frame(self):Show()
end
