local AddonName, GGL = ...
local Templates = GGL.Templates

local PriceSource = GGL.PriceSource

local ItemPriceSet, ItemPriceSetHelper = Templates:Register("ItemPriceSet")

-- lua

-- wow
local GetItemInfo = GetItemInfo
local ITEM_QUALITY_COLORS = _G.ITEM_QUALITY_COLORS
local GetPetInfoByItemID = C_PetJournal and  C_PetJournal.GetPetInfoByItemID or false

-- GGL
local ALWAYS_USE_VENDOR_PRICE_FOR_ITEMID = GGL.ALWAYS_USE_VENDOR_PRICE_FOR_ITEMID
local GetVendorPrice = GGL.PriceSource:GetFunction()


-- ##################
-- local
-- ##################
local NUM_QUALITYS = #ITEM_QUALITY_COLORS

-- ##################
-- static
-- ##################
local Static = {}
Templates.ItemPriceSet = Static

-- ##################
-- template
-- ##################
function ItemPriceSet:Constructor(setKey, isSettingString, sessionPriceLogTable)
    -- set baisc settings with default price source
    if isSettingString then
        self:LoadSetSettings(setKey, sessionPriceLogTable)
    else
        self:LoadSet(setKey, sessionPriceLogTable)
    end
end

function ItemPriceSet:LoadSet(setKey, sessionPriceLogTable)
    local db = GGL.db.Config.Sets[GGL.SET_ID_LIST.PriceSource]
    self.setKey = setKey
    if db and db[setKey] then
        local pass, db = GGL:Deserialize(db[setKey].setting)
        self:SetFilterDB(db)
    else
        self:SetFilterDB()
    end
    self.sessionPriceLogTable = sessionPriceLogTable
end

function ItemPriceSet:LoadSetSettings(setting, sessionPriceLogTable)
    if setting then
        local pass, db = GGL:Deserialize(setting)
        self:SetFilterDB(db)
    else
        self:SetFilterDB()
    end
    self.sessionPriceLogTable = sessionPriceLogTable
end

function ItemPriceSet:ReloadSet()
    if self.setKey then
        self:LoadSet(self.setKey)
    end
end

function ItemPriceSet:SetFilterDB(db)
    db = db or {}
    self.db = db
    local price = {}
    for qualityID = 0, #GGL.ITEM_QUALITY_INFO do
        local quality = GGL.ITEM_QUALITY_INFO[qualityID]
        price[qualityID] = GGL.PriceSource:GetFunction(db[qualityID])
        price["v"..qualityID] = db["v"..qualityID]
    end
    if db["s"] then
        price["s"] = GGL.PriceSource:GetFunction(db["s"])
    end
    self.price = price
end

--- Returns the price in copper
function ItemPriceSet:GetPrice(itemID, itemString)
    if self.sessionPriceLogTable and (self.sessionPriceLogTable[itemString] or self.sessionPriceLogTable[itemID]) then
        return (self.sessionPriceLogTable[itemString] or self.sessionPriceLogTable[itemID]) or 0
    end
    local _, _, quality, _, _, _, _, _, _, _, _, itemClassID, itemSubClassID, bindType = GetItemInfo(itemID)
    -- not cached
    if not quality then
        return nil
    end
    local priceValue, basePrice = nil, 0

    local price = self.price

    if itemID and ALWAYS_USE_VENDOR_PRICE_FOR_ITEMID[itemID] then
        basePrice, priceValue = GetVendorPrice(itemID, itemString)
    elseif bindType == 1 then
        if GetPetInfoByItemID and itemClassID == 15 and itemSubClassID == 2 and price[quality] then
            local _, _, _, _, _, _, _, _, _, _, _, _, speciesID = GetPetInfoByItemID(itemID)
            if speciesID then
                basePrice, priceValue = price[quality](itemID, itemString)
            end
        end
        if not priceValue and price["s"] then
            basePrice, priceValue = price["s"](itemID, itemString)
        end
    else
        if itemString and price["v"..quality] then
            basePrice, priceValue = price[quality](itemID, itemString)
        else
            priceValue = price[quality](itemID)
        end
    end

    if itemString and price["v"..quality] then
        self.sessionPriceLogTable[itemString] = priceValue or basePrice
    else
        self.sessionPriceLogTable[itemID] = priceValue or 0
    end

    return priceValue or 0
end

function ItemPriceSet:RawGoldAllowed()
    return self.db["r"] == true and false or true
end

function ItemPriceSet:QualityUseVariations(qualityID)
    return self.price["v"..qualityID]
end

function ItemPriceSet:LoggedItemUseVariations(itemID, itemString)
    if self.sessionPriceLogTable[itemString] then
        return true
    elseif self.sessionPriceLogTable[itemID] then
        return false
    end
    return nil
end